@@ -23,110 +23,123 @@
 //!}
 //!```
 //!
-//! We can use a [`RecRef`] directly
-//! ----------------------------------------------
+//!We can use a [`RecRef`] directly
+//!----------------------------------------------
 //!```rust
+//!use recursive_reference::*;
+//!
 //! # enum List<T> {
-//! # Root(Box<Node<T>>),
-//! # Empty,
+//! #    Root(Box<Node<T>>),
+//! #    Empty,
 //! # }
 //! # struct Node<T> {
-//! # value: T,
-//! # next: List<T>,
+//! #    value: T,
+//! #    next: List<T>,
 //! # }
-//! use recursive_reference::*;
 //!
-//! fn main() -> Result<(), ()> {
-//!     let node1 = Node { value : 5, next : List::Empty };
-//!     let mut node2 = Node { value : 2, next : List::Root(Box::new(node1)) };
+//!fn main() -> Result<(), ()> {
+//!   // crate a list to test
+//!   let node1 = Node {
+//!       value: 5,
+//!       next: List::Empty,
+//!   };
+//!   let mut node2 = Node {
+//!       value: 2,
+//!       next: List::Root(Box::new(node1)),
+//!   };
 //!
-//!     let mut rec_ref = RecRef::new(&mut node2);
-//!     assert_eq!(rec_ref.value, 2); // rec_ref is a smart pointer to the current node
-//!     rec_ref.value = 7; // change the value at the head of the list
-//!     RecRef::extend_result(&mut rec_ref, |node| match &mut node.next {
-//!         List::Root(next_node) => Ok(next_node),
-//!         List::Empty => Err(()),
-//!     })?;
-//!     assert_eq!(rec_ref.value, 5);
-//!     // extend the RecRef
-//!     let res = RecRef::extend_result(&mut rec_ref, |node| match &mut node.next {
-//!         List::Root(next_node) => Ok(next_node),
-//!         List::Empty => Err(()),
-//!     });
-//!     assert_eq!(res, Err(())); // could not go forward because it reached the end of the list
-//!     assert_eq!(rec_ref.value, 5);
-//!     let last = RecRef::pop(&mut rec_ref).ok_or(())?;
-//!     assert_eq!(last.value, 5);
-//!     assert_eq!(rec_ref.value, 7) ; // moved back to the head of the list because we popped rec_ref
-//!     Ok(())
-//! }
+//!   // create a `RecRef`
+//!   let mut rec_ref = RecRef::new(&mut node2);
+//!   // rec_ref is a smart pointer to the current node
+//!   assert_eq!(rec_ref.value, 2);
+//!
+//!   // move forward through the list
+//!   RecRef::extend_result(&mut rec_ref, |node| match &mut node.next {
+//!       List::Root(next_node) => Ok(next_node),
+//!       List::Empty => Err(()),
+//!   })?;
+//!   assert_eq!(rec_ref.value, 5); // now we're at the second node
+//!
+//!   // pop the `RecRef`, moving it back to the head
+//!   RecRef::pop(&mut rec_ref).ok_or(())?;
+//!   assert_eq!(rec_ref.value, 2);
+//!   Ok(())
+//!}
 //!```
 //!
-//! We can also wrap a [`RecRef`] in a walker struct
-//! ----------------------------------------------
-//! Note: this time we are using a `RecRef<List<T>>` and not a `RecRef<Node<T>>`, to allow pointing
-//! at the empty end of the list.
+//!We can also wrap a [`RecRef`] in a walker struct
+//!----------------------------------------------
+//!Note: this time we are using a `RecRef<List<T>>` and not a `RecRef<Node<T>>`, to allow pointing
+//!at the empty end of the list.
 //!```rust
+//!use recursive_reference::*;
 //! # enum List<T> {
-//! # Root(Box<Node<T>>),
-//! # Empty,
+//! #    Root(Box<Node<T>>),
+//! #    Empty,
 //! # }
 //! # struct Node<T> {
-//! # value: T,
-//! # next: List<T>,
+//! #    value: T,
+//! #    next: List<T>,
 //! # }
-//! use recursive_reference::*;
-//! struct Walker<'a, T> {
-//!     rec_ref : RecRef<'a, List<T>>
-//! }
-//! impl<'a, T> Walker<'a, T> {
-//!     pub fn new(list: &'a mut List<T>) -> Self {
-//!         Walker {
-//!             rec_ref : RecRef::new(list)
-//!         }
-//!     }
+//!struct Walker<'a, T> {
+//!   rec_ref: RecRef<'a, Node<T>>,
+//!}
+//!impl<'a, T> Walker<'a, T> {
+//!   /// Crates a new Walker
+//!   pub fn new(node: &'a mut Node<T>) -> Self {
+//!       Walker {
+//!           rec_ref: RecRef::new(node),
+//!       }
+//!   }
 //!
-//!     /// Returns `None` when at the tail end of the list
-//!     pub fn next(&mut self) -> Option<()> {
-//!         RecRef::extend_result(&mut self.rec_ref, |current| match current {
-//!             List::Empty => Err(()),
-//!             List::Root(node) => Ok(&mut node.next),
-//!         }).ok()
-//!     }
+//!   /// Returns `None` when at the tail end of the list.
+//!   /// Moves to the next node.
+//!   pub fn next(&mut self) -> Option<()> {
+//!       RecRef::extend_result(&mut self.rec_ref, |current| match &mut current.next {
+//!           List::Empty => Err(()),
+//!           List::Root(node) => Ok(node),
+//!       })
+//!       .ok()
+//!   }
 //!
-//!     /// Returns `None` when at the head of the list
-//!     pub fn prev(&mut self) -> Option<()> {
-//!         RecRef::pop(&mut self.rec_ref)?;
-//!         Some(())
-//!     }
+//!   /// Returns `None` when at the head of the list.
+//!   /// Goes back to the previous node.
+//!   pub fn prev(&mut self) -> Option<()> {
+//!       RecRef::pop(&mut self.rec_ref)?;
+//!       Some(())
+//!   }
 //!
-//!     /// Returns `None` when at the tail end of the list
-//!     pub fn value_mut(&mut self) -> Option<&mut T> {
-//!         match &mut *self.rec_ref {
-//!             List::Root(node) => Some(&mut node.value),
-//!             List::Empty => None,
-//!         }
-//!     }
-//! }
+//!   /// Returns `None` when at the tail end of the list.
+//!   /// Returns `Some(reference)` where `reference` is a mutqable reference to the current value.
+//!   pub fn value_mut(&mut self) -> &mut T {
+//!       &mut self.rec_ref.value
+//!   }
+//!}
 //!
-//! fn main() -> Result<(), ()> {
-//!     let node1 = Node { value : 5, next : List::Empty };
-//!     let node2 = Node { value : 2, next : List::Root(Box::new(node1)) };
-//!     let mut list = List::Root(Box::new(node2));
+//!fn main() -> Result<(), ()> {
+//!   // crate a list to test
+//!   let node1 = Node {
+//!       value: 5,
+//!       next: List::Empty,
+//!   };
+//!   let mut node2 = Node {
+//!       value: 2,
+//!       next: List::Root(Box::new(node1)),
+//!   };
 //!
-//!     let mut walker = Walker::new(&mut list);
-//!     assert_eq!(walker.value_mut().cloned(), Some(2));
-//!     *walker.value_mut().ok_or(())? = 7;
-//!     walker.next().ok_or(())?;
-//!     assert_eq!(walker.value_mut().cloned(), Some(5));
-//!     walker.next().ok_or(())?;
-//!     assert_eq!(walker.value_mut().cloned(), None); // end of the list
-//!     walker.prev().ok_or(())?;
-//!     assert_eq!(walker.value_mut().cloned(), Some(5));
-//!     walker.prev().ok_or(())?;
-//!     assert_eq!(walker.value_mut().cloned(), Some(7)); // we changed the value at the head
-//!     Ok(())
-//! }
+//!   // create a walker for the list
+//!   let mut walker = Walker::new(&mut node2);
+//!   // walker has mutable access to the node value
+//!   assert_eq!(*walker.value_mut(), 2);
+//!   // move to the next node
+//!   walker.next().ok_or(())?;
+//!   assert_eq!(*walker.value_mut(), 5);
+//!   assert_eq!(walker.next(), None); // currently at the end of the list
+//!                                    // move back
+//!   walker.prev().ok_or(())?;
+//!   assert_eq!(*walker.value_mut(), 2);
+//!   Ok(())
+//!}
 //!```
 //! With a [`RecRef`] you can
 //! ----------------------------------------------
@@ -155,14 +168,14 @@
 //! Another important point to consider is the safety of
 //! the actual call to [`extend`][RecRef::extend]: see its documentation.
 #![no_std]
-#![doc(html_root_url = "https://docs.rs/recursive_reference/0.2.0/recursive_reference/")]
+#![doc(html_root_url = "https://docs.rs/recursive_reference/0.3.0/recursive_reference/")]
 
 extern crate alloc;
 use alloc::vec::*;
 
 use core::marker::PhantomData;
-use core::ops::Deref;
-use core::ops::DerefMut;
+use core::ops::{Deref, DerefMut};
+use core::ptr::NonNull;
 use void::ResultVoidExt;
 
 /// A Recursive reference.
@@ -186,24 +199,16 @@ use void::ResultVoidExt;
 ///
 /// Internally, the [`RecRef`] stores a [`Vec`] of pointers, that it extends and pops from.
 pub struct RecRef<'a, T: ?Sized> {
-    head: *mut T,
-    vec: Vec<*mut T>,
+    head: NonNull<T>,
+    vec: Vec<NonNull<T>>,
     phantom: PhantomData<&'a mut T>,
 }
 
-// TODO: consider converting the pointers to values without checking for null values.
-// it's supposed to work, since the pointers only ever come from references.
-// otherwise, when 1.53 rolls out, convert to `NonNull`.
-
-// these aren't ever supposed to happen. but since we touch unsafe code, we might as well
-// have clear error message when we `expect()`
-const NULL_POINTER_ERROR: &str = "error! somehow got null pointer";
-
 impl<'a, T: ?Sized> RecRef<'a, T> {
     /// Creates a new RecRef containing only a single reference.
     pub fn new(r: &'a mut T) -> Self {
         RecRef {
-            head: r as *mut T,
+            head: NonNull::from(r),
             vec: Vec::new(),
             phantom: PhantomData,
         }
@@ -284,7 +289,7 @@ impl<'a, T: ?Sized> RecRef<'a, T> {
         // However, that is the "most correct" lifetime - the reference's actual lifetime may
         // be anything up to `'a`,
         // depending on whether the user will pop it earlier than that.
-        let head_ref: &'a mut T = unsafe { rec_ref.head.as_mut() }.expect(NULL_POINTER_ERROR);
+        let head_ref: &'a mut T = unsafe { rec_ref.head.as_mut() };
 
         match func(head_ref, PhantomData) {
             Ok(p) => {
@@ -325,11 +330,11 @@ impl<'a, T: ?Sized> RecRef<'a, T> {
         // However, that is the "most correct" lifetime - the reference's actual lifetime may
         // be anything up to `'a`,
         // depending on whether the user will pop it earlier than that.
-        let head_ref: &'a mut T = unsafe { rec_ref.head.as_mut() }.expect(NULL_POINTER_ERROR);
+        let head_ref: &'a mut T = unsafe { rec_ref.head.as_mut() };
 
         match func(head_ref, PhantomData) {
             Ok(p) => {
-                rec_ref.head = p as *mut T;
+                rec_ref.head = NonNull::from(p);
                 Ok(())
             }
             Err(e) => Err(e),
@@ -342,7 +347,7 @@ impl<'a, T: ?Sized> RecRef<'a, T> {
     /// right type.
     pub fn push(rec_ref: &mut Self, r: &'a mut T) {
         rec_ref.vec.push(rec_ref.head);
-        rec_ref.head = r as *mut T;
+        rec_ref.head = NonNull::from(r);
 
         /* alternative definition using a call to `extend_result_precise`.
         // in order to name 'x, replace the signature with:
@@ -364,7 +369,7 @@ impl<'a, T: ?Sized> RecRef<'a, T> {
     /// If the [`RecRef`] has only one reference left, this returns `None`, because
     /// the [`RecRef`] can't be empty.
     pub fn pop(rec_ref: &mut Self) -> Option<&mut T> {
-        let res = unsafe { rec_ref.head.as_mut() }.expect(NULL_POINTER_ERROR);
+        let res = unsafe { rec_ref.head.as_mut() };
         rec_ref.head = rec_ref.vec.pop()?; // We can't pop the original reference. In that case, Return None.
         Some(res)
     }
@@ -374,8 +379,8 @@ impl<'a, T: ?Sized> RecRef<'a, T> {
     /// * This will consume the [`RecRef`]
     /// * [`Self::pop`] will never pop the first original reference, because that would produce an
     ///   invalid [`RecRef`]. [`Self::into_ref`] will.
-    pub fn into_ref(rec_ref: Self) -> &'a mut T {
-        unsafe { rec_ref.head.as_mut() }.expect(NULL_POINTER_ERROR)
+    pub fn into_ref(mut rec_ref: Self) -> &'a mut T {
+        unsafe { rec_ref.head.as_mut() }
     }
 }
 
@@ -385,7 +390,7 @@ impl<'a, T: ?Sized> RecRef<'a, T> {
 impl<'a, T: ?Sized> Deref for RecRef<'a, T> {
     type Target = T;
     fn deref(&self) -> &T {
-        unsafe { self.head.as_ref() }.expect(NULL_POINTER_ERROR)
+        unsafe { self.head.as_ref() }
     }
 }
 
@@ -394,7 +399,7 @@ impl<'a, T: ?Sized> Deref for RecRef<'a, T> {
 /// Therefore, it implements `Deref` and `DerefMut` with `Item=T`.
 impl<'a, T: ?Sized> DerefMut for RecRef<'a, T> {
     fn deref_mut(&mut self) -> &mut T {
-        unsafe { self.head.as_mut() }.expect(NULL_POINTER_ERROR)
+        unsafe { self.head.as_mut() }
     }
 }
 
